﻿using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XNAWinformReversi.GameLogic;

namespace XNAWinformReversi.CustomControls
{
    public class BoardDisplay: WinFormsGraphicsDevice.GraphicsDeviceControl
    {
        enum Piece { Empty, White, Black };
        private ContentManager content;
        private SpriteBatch spriteBatch;
        public AnimatedFlip animatedFlip;
        private Texture2D blkFlip;
        private Texture2D whtFlip;
        private Texture2D whtPiece;
        private Texture2D blkPiece;
        private Texture2D blackPlaced;
        private Texture2D whitePlaced;
        private Texture2D board;

        public int[,] baseBoardArray;
        public Square placedPiece;
        public int placedPieceColor;
        public bool showPiecePlaced;
        public List<Square> flipList;

        protected override void Initialize()
        {
            content = new ContentManager(Services, "Content");
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load Content to be used.
            blkFlip = content.Load<Texture2D>("blackToWhiteFlipSheet");
            whtFlip = content.Load<Texture2D>("whiteToBlackFlipSheet");
            animatedFlip = new AnimatedFlip(blkFlip, 10, 4);
            blkPiece = content.Load<Texture2D>("blackImg0");
            whtPiece = content.Load<Texture2D>("whiteImg0");
            blackPlaced = content.Load<Texture2D>("blackPlaced");
            whitePlaced = content.Load<Texture2D>("whitePlaced");
            board = content.Load<Texture2D>("Burghy-Reversi-Board");
            flipList = new List<Square>();
            baseBoardArray = new int[8, 8];
        }

        public void setUpFlip(int color)
        {
            // set texture for black or white flip
            if(color == (int)Piece.Black)
                animatedFlip = new AnimatedFlip(whtFlip, 10, 4);
            else if(color == (int)Piece.White)
                animatedFlip = new AnimatedFlip(blkFlip, 10, 4);
        }

        public void resetBaseBoardArray()
        {
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    baseBoardArray[x, y] = 0;
                }
            }
        }

        protected override void Draw()
        {
            GraphicsDevice.Clear(Color.DarkGreen);
           
            spriteBatch.Begin();

            spriteBatch.Draw(board, new Vector2(0, 0), Color.White);

            
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    int temp = baseBoardArray[x, y];
                    if (temp == (int)Piece.Black)
                    {
                        spriteBatch.Draw(blkPiece, new Vector2((x*50)+25, (y*50)+25), Color.White);
                    }
                    else if (temp == (int)Piece.White)
                    {
                        spriteBatch.Draw(whtPiece, new Vector2((x*50)+25, (y*50)+25), Color.White);
                    }
                }
            }
            if (showPiecePlaced)
            {
                if(placedPieceColor == (int)Piece.Black)
                    spriteBatch.Draw(blackPlaced, new Vector2((placedPiece.x * 50)+25, (placedPiece.y * 50)+25), Color.White);
                else if(placedPieceColor == (int)Piece.White)
                    spriteBatch.Draw(whitePlaced, new Vector2((placedPiece.x * 50)+25, (placedPiece.y * 50)+25), Color.White);
            }
            spriteBatch.End();

            foreach (Square temp in flipList)
            {
                animatedFlip.Draw(spriteBatch, new Vector2((temp.x*50)+25, (temp.y*50)+25));
            }
            
        }
    }
}
