﻿using System;
using System.Collections.Generic;
using XNAWinformReversi.UI_Forms;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;

namespace XNAWinformReversi.GameLogic
{
    public class Controller
    {

        enum Piece { Empty, White, Black };
        enum Player { User, Computer };
        private int _blackPlayer;
        private int _whitePlayer;
        private MainUIForm _form;
        public BoardModel _model;
        private bool _blkCanMove = false;
        private bool _whtCanMove = false;
        private EasyAI _blackAI;
        private EasyAI _whiteAI;
        
        public Controller(MainUIForm form, int blkPlayer, int whtPlayer)
        {
            _form = form;
            _blackPlayer = blkPlayer;
            _whitePlayer = whtPlayer;  
        }

        public void continueGame()
        {
            setPieceCount();
            updateWhosTurn();

            if (_form.gameInProgress)
            {
                if (_model.getCurrentPlayer() == (int)Piece.Black)
                {
                    _blkCanMove = _model.loadLegalMoves((int)Piece.Black);

                    if (!_blkCanMove)
                    {
                        _blkCanMove = false;
                        _model.setCurrentPlayer((int)Piece.White);
                        checkGameOver();
                        updateWhosTurn();
                    }
                    else if (_blackPlayer == (int)Player.Computer)
                    {
                        Square temp = _blackAI.getComputerMove();
                        _form.startMove((int)Piece.Black, temp.x, temp.y);
                    }
                }

                else if (_model.getCurrentPlayer() == (int)Piece.White)
                {

                    _whtCanMove = _model.loadLegalMoves((int)Piece.White);

                    if (!_whtCanMove)
                    {
                        _whtCanMove = false;
                        _model.setCurrentPlayer((int)Piece.Black);
                        checkGameOver();
                        updateWhosTurn();
                    }
                    else if (_whitePlayer == (int)Player.Computer)
                    {
                        Square temp = _whiteAI.getComputerMove();
                        _form.startMove((int)Piece.White, temp.x, temp.y);
                    }

                }
            }
                
        }

        public void handleMouseClicks(int clickedX, int clickedY)
        {

            if (_model.getCurrentPlayer() == (int)Piece.Black && _blackPlayer == (int)Player.User)
            {
                foreach (Square temp in _model.legalMovesList)
                {
                    if (temp.x == clickedX && temp.y == clickedY)
                    {
                        _form.startMove((int)Piece.Black, clickedX, clickedY);
                        
                    }
                }
            }
            else if (_model.getCurrentPlayer() == (int)Piece.White && _whitePlayer == (int)Player.User)
            {
                foreach (Square temp in _model.legalMovesList)
                {
                    if (temp.x == clickedX && temp.y == clickedY)
                    {
                        _form.startMove((int)Piece.White, clickedX, clickedY);

                    }
                }
            }

        }

        private void checkGameOver()
        {
            if (!_blkCanMove && !_whtCanMove)
            {
                doGameOver();
            }
            else
            {
                continueGame();
            }
        }

        private void doGameOver()
        {
            int winner = _model.getWinner();
            string winStr = "";
            if (winner == 0)
            {
                winStr = "--- TIE ---";
            }
            else if (winner == (int)Piece.Black)
            {
                winStr = "--- Black Wins ---";
            }
            else
            {
                winStr = "--- White Wins ---";
            }

            _form.doWin(winStr);
            
        }

        public void newGame()
        {
            _model = new BoardModel();
            _model.newGame();
            setPieceCount();
            _form.boardDisplay1.baseBoardArray = _model._board2DArray;
            _form.boardDisplay1.Invalidate();
            if (_blackPlayer == (int)Player.Computer)
                _blackAI = new EasyAI((int)Piece.Black, _model);
            if(_whitePlayer == (int)Player.Computer)
                _whiteAI = new EasyAI((int)Piece.White, _model);
            continueGame();
        }

        private void updateWhosTurn()
        {
            if (_model.getCurrentPlayer() == (int)Piece.Black)
                _form.playerTurn = (int)Piece.Black;
            else
                _form.playerTurn = (int)Piece.White;
            _form.Invalidate();
        }

        private void setPieceCount()
        {
            _model.loadPieceCount();
            _form.blackCount.Text = "Black: " + _model.getBlackPieceCount();
            _form.whiteCount.Text = "White: " + _model.getWhitePieceCount();
        }

    }
}
