﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using XNAWinformReversi.UIForms;
using XNAWinformReversi.GameLogic;
using System.Drawing;

namespace XNAWinformReversi.UI_Forms
{
    public partial class MainUIForm : Form
    {
        enum Piece { Empty, White, Black };
        enum Player { User, Computer};
        private int count1 = 0;
        private int count2 = 0;
        PlayerForm playerForm;
        MustResignFirstForm resignFirstForm;
        public bool gameInProgress;
        public int playerTurn = (int)Piece.Black;
        private int _blkPlayer;
        private int _whtPlayer;
        Controller controller;
        private int[,] _boardArrayMinusFlips;
        private int _color;
        private int _clickedX;
        private int _clickedY;
        private bool clicksDisabled = false;

        Pen whtPen = new Pen(Color.White, 2);
        Pen blkPen = new Pen(Color.Black, 2);

        public MainUIForm()
        {
            InitializeComponent();
            playerForm = new PlayerForm();
            resignFirstForm = new MustResignFirstForm();
            gameInProgress = false;
            hideComponents();

            // construction area
            _blkPlayer = (int)Player.User;
            _whtPlayer = (int)Player.Computer;
            _boardArrayMinusFlips = new int[8, 8];
        }

        #region timers

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (count1 < 39)
            {
                boardDisplay1.animatedFlip.Update();
                boardDisplay1.Invalidate();
            }
            else
            {
                timer1.Stop();
                boardDisplay1.showPiecePlaced = false;
                finishMove();
                boardDisplay1.Invalidate();
            }
            count1++;
        }

        private void timer2_Tick(object sender, EventArgs e)
        {
            if (count2 > 4)
            {
                timer2.Stop();
                handleFlips();
            }
            count2++;
        }

        # endregion

        # region move animation methods

        public void startMove(int color, int clickedX, int clickedY)
        {
            clicksDisabled = true;
            _color = color;
            _clickedX = clickedX;
            _clickedY = clickedY;

            boardDisplay1.showPiecePlaced = true;
            boardDisplay1.placedPieceColor = color;
            boardDisplay1.placedPiece = new Square(clickedX, clickedY);
            boardDisplay1.Invalidate();
            count2 = 0;
            timer2.Start();
        }

        private void handleFlips()
        {
            
            controller._model.loadPiecesToFlip(_color, _clickedX, _clickedY);
            _boardArrayMinusFlips = controller._model._board2DArray;

            foreach (Square item in controller._model.piecesToFlipList)
            {
                _boardArrayMinusFlips[item.x, item.y] = (int)Piece.Empty;
            }

            boardDisplay1.baseBoardArray = _boardArrayMinusFlips;
            
            boardDisplay1.setUpFlip(_color);
            boardDisplay1.Invalidate();
            boardDisplay1.flipList = controller._model.piecesToFlipList;
            count1 = 0;
            timer1.Start();
            
        }

        private void finishMove()
        {
            foreach (Square item in controller._model.piecesToFlipList)
            {
                controller._model._board2DArray[item.x, item.y] = _color;
            }

            controller._model._board2DArray[_clickedX, _clickedY] = _color;
            int opponent = controller._model.getOpponentColor(_color);
            controller._model.setCurrentPlayer(opponent);
            playerTurn = opponent;
            this.Invalidate();
            boardDisplay1.Invalidate();
            clicksDisabled = false;
            controller.continueGame();
        }

        # endregion

        private void boardDisplay1_Click(object sender, MouseEventArgs e)
        {
            int clickedX = (e.X-25) / 50;
            int clickedY = (e.Y-25) / 50;

            if (gameInProgress && !clicksDisabled)
                controller.handleMouseClicks(clickedX, clickedY);
        }

        private void MainUIForm_Paint(object sender, PaintEventArgs e)
        {
            if (playerTurn == (int)Piece.Black && gameInProgress)
            {
                int blkX = blackLabel.Location.X;
                int blkY = blackLabel.Location.Y;
                Rectangle rect = new Rectangle(blkX - 5, blkY - 5, blackLabel.Width + 10, blackLabel.Height + 10);
                e.Graphics.DrawRectangle(blkPen, rect);
            }
            else if (playerTurn == (int)Piece.White && gameInProgress)
            {
                int whtX = whiteLabel.Location.X;
                int whtY = whiteLabel.Location.Y;
                Rectangle rect = new Rectangle(whtX - 5, whtY - 5, whiteLabel.Width + 10, whiteLabel.Height + 10);
                e.Graphics.DrawRectangle(whtPen, rect);
            }
        }

        #region toolstrip region

        private void playerSetupToolStripMenuItem_Click(object sender, EventArgs e)
        {

            if (!gameInProgress)
            {
                if (playerForm.ShowDialog() == DialogResult.OK)
                {
                    // get choice for black
                    if (playerForm.player1Box.SelectedIndex == 0)
                    {
                        _blkPlayer = (int)Player.User;
                    }
                    else if (playerForm.player1Box.SelectedIndex == 1)
                    {
                        _blkPlayer = (int)Player.Computer;
                    }

                    // get choice for white
                    if (playerForm.player2Box.SelectedIndex == 0)
                    {
                        _whtPlayer = (int)Player.User;
                    }
                    else if (playerForm.player2Box.SelectedIndex == 1)
                    {
                        _whtPlayer = (int)Player.Computer;
                    }

                }
            }
            else
            {
                resignFirstForm.ShowDialog();
            }
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!gameInProgress)
            {
                newGame();
            }
            else if (gameInProgress)
            {
                resignFirstForm.ShowDialog();
            }
        }

        private void resignToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string resign = "Game Resigned";
            playerTurn = 0;
            this.Invalidate();
            doWin(resign);
        }

        #endregion

        private void newGame()
        {
            showComponents();
            winLabel.Visible = false;
            gameInProgress = true;
            boardDisplay1.resetBaseBoardArray();
            boardDisplay1.flipList.Clear();
            controller = new Controller(this, _blkPlayer, _whtPlayer);
            controller.newGame();
            boardDisplay1.Invalidate();
        }

        public void doWin(string winStr)
        {
            gameInProgress = false;
            whiteLabel.Visible = false;
            blackLabel.Visible = false;
            winLabel.Visible = true;
            winLabel.Text = winStr;
        }

        private void hideComponents()
        {
            whiteLabel.Visible = false;
            blackLabel.Visible = false;
            blackCount.Visible = false;
            whiteCount.Visible = false;
            boardDisplay1.Visible = true;
            panel1.Visible = false;
            winLabel.Visible = false;
        }

        private void showComponents()
        {
            whiteLabel.Visible = true;
            blackLabel.Visible = true;
            blackCount.Visible = true;
            whiteCount.Visible = true;
            boardDisplay1.Visible = true;
            panel1.Visible = true;
        }

    }
}
